use Tk;
use Tk::Label;
use Tk::Entry;
use Tk::Button;
use Tk::Text;
use RXParser;
use locale;

sub expandpath {
	+($1, $2, $3) if $_[0]=~/^(.+?)([\\\/])([^\\\/]+)$/o
}

sub filenameValid {+shift=~/^[\w\/\:\.\-]+$/o}
sub fileExists {
	my$file=shift;
	return &filenameValid($file) && -f $file
}

sub formatFile {
	my($in, $out, $tpl, $rules)=(shift, shift, shift, shift);
	+RXParser::RXML->new(
				parser=>RXParser::Parser->new(
						mapping=>RXParser::Mapping->new(
									file=>$rules
						)->prepare,
						file=>$in
				)->prepare,
				file=>$tpl
	)->prepare->saveAs($out)
}

$line_height=1;

@fields=(
	'in',
	'out',
	'tpl',
	'rules'
);

$top = new MainWindow(
	-title=>'GRXParser',
	-width=>200,
	-height=>200
);

foreach(@fields) {
	${+'frame_'.$_}=$top->Frame->pack(
		-anchor=>'n',
		-fill=>'x',
		-side=>top,
		-expand=>yes
	);

	${+'label_'.$_}=${+'frame_'.$_}->Label(
		-width=>20,
		-justify=>'right'
	)->pack(
		-side=>left,
		-fill=>'x',
		-anchor=>nw
	);

	${+'entry_'.$_}=${+'frame_'.$_}->Entry(
		-textvariable=>+\${+'file_'.$_},
		-width=>50
	)->pack(
		-side=>left,
		-fill=>'x',
		-expand=>yes,
		-anchor=>nw
	);

	${+'button_'.$_}=${+'frame_'.$_}->Button(
		-text=>'..'
	)->pack(
		-side=>left,
		-anchor=>nw
	);
}

$pad=$top->Frame->pack(
	-expand=>yes,
	-fill=>'x'
);

$button_show=$pad->Button(
	-text=>'  ',
	-wraplength=>100,
	-height=>2,
	-command=>sub{
		if(&fileExists($file_out)) {
			my$fh;
			open $fh, $file_out;
				$text->insert('end', $_) while <$fh>;
			close $fh;
		}
	}
)->pack(
	-side=>'left',
	-anchor=>'center'
);

$button_start=$pad->Button(
	-text=>'',
	-command=>sub{
		return $top->messageBox(-message=>'   ', -title=>'')
			unless &fileExists($file_in);
		return $top->messageBox(-message=>'   ', -title=>'')
			unless &filenameValid($file_out);
		return $top->messageBox(-message=>'   ', -title=>'')
			unless &filenameValid($file_tpl);
		return $top->messageBox(-message=>'    ', -title=>'')
			unless &filenameValid($file_rules);
		if(&formatFile($file_in, $file_out, $file_tpl, $file_rules)) {
			$top->messageBox(-message=>'  ', -title=>'');
		} else {
			$top->messageBox(-message=>' ', -title=>'');
		}
	},
	-height=>2
)->pack(
	-side=>'left',
	-anchor=>'center'
);

$text=$top->Text->pack(
	-fill=>'both',
	-expand=>'yes',
	-anchor=>nw,
	-side=>top
);

$button_in->configure(
	-command=>sub{
			my($dir, $sep, $file)=&expandpath($file_in);
			my$result=$top->getOpenFile(
				-title=>'   ',
				-initialfile=>$file,
				-initialdir=>$dir,
				-filetypes=>[[' '=>'*.*']]
			);
			$file_in=$result if $result;
		}
);

$button_out->configure(
	-command=>sub{
		my($dir, $sep, $file)=&expandpath($file_out);
		my$result=$top->getSaveFile(
			-title=>'   ',
			-initialfile=>$file,
			-initialdir=>$dir,
			-filetypes=>[[' '=>'*.*']]
		);
		$file_out=$result if $result;
	}
);

$button_tpl->configure(
	-command=>sub{
		my($dir, $sep, $file)=&expandpath($file_tpl);
		my$result=$top->getOpenFile(
			-title=>'   ',
			-initialfile=>$file,
			-initialdir=>$dir,
			-filetypes=>[
				[' '=>'*.rxml'],
				[' '=>'*.*']
			]
		);
		$file_tpl=$result if $result;
	}
);

$button_rules->configure(
	-command=>sub{
		my($dir, $sep, $file)=&expandpath($file_rules);
		my$result=$top->getOpenFile(
			-title=>'    ',
			-initialfile=>$file,
			-initialdir=>$dir,
			-filetypes=>[
				['  '=>'*.rx'],
				[' '=>'*.*']
			]
		);
		$file_rules=$result if $result;
	}
);

$label_in->configure(
	-text=>' :'
);
$label_out->configure(
	-text=>' :'
);
$label_tpl->configure(
	-text=>':'
);
$label_rules->configure(
	-text=>' :'
);

MainLoop;
