package fileops;

use constant OS=>$^O!~/MS|Win|Windows|Win32|Microsoft/o;

sub osflock {+ !OS || OS && flock $_[0], $_[1]}
sub osopen  {
	my($file, $mode, $fh)=(shift, shift || 4);
	open($fh, $file) && &osflock($fh, $mode) && binmode $fh or die $!;
	+$fh
}

sub osclose {
	my($fh, $mode)=(shift, shift || 8);
	+ &osflock($fh, $mode) && close $fh or die $!
}

sub osreadfile {
	my($file, $fh)=shift;
	$fh=&osopen($file) or die $!;
		my$old=$/;$/=undef;
		my$str=<$fh>;$/=$old;
	&osclose($fh) or die $!;
	+$str
}

package RXParser;

sub new {
	my$class=shift;
	my$self={@_};
	bless $self, $class;
	+$self
}

sub DESTROY {undef $_[0]}

package RXParser::Filter;
use Entities qw'decode_entities';

BEGIN {
	%MODE=(
		'HTML'=>sub{+&decode_entities(shift)},
		'TEXT'=>sub{+&decode_entities($MODE{'TEXT/ENTITY'}->(shift))},
		'TEXT/ENTITY'=>sub{
			my$str=shift;
			$str=~s/<[^>]+>//gos;
			+$str
		},
		'DATE'=>sub{
			my$date=shift;
			return $date unless $date=~/(\d{1,2})\D+(\d{1,2})\D+(\d{4}|\d{2})/o;
			+sprintf '%02d.%02d.%04d', $1, $2, $3<40?$3+2000:$3
		},
		'LINK'=>sub{+$MODE{'TEXT/ENTITY'}->(shift)}
	);
}

sub prepare {
	my($self, $str, $mode)=(shift, shift, uc shift);
	+ ref $MODE{$mode} eq 'CODE' ? $MODE{$mode}->($str) : $str
}

sub trim {+$_[0]=~/^\s*(.*?)\s*$/os? $1 : $_[0]}

package RXParser::Mapping;
use base 'RXParser';

sub prepare {
	my$self=shift;
	my($vi, $vp)=(1, 'v%d');
	my$fh=&fileops::osopen(shift || $self->{file} || return undef);
	$self->{result}={};

		while(<$fh>) {
			next unless index $_, '#';
			if(/^\s*(['"]?)(.+?)\1\s*(?:\:\s*(.+?))?\s*\((.+?)\)\s*(.)[\r\n]*$/om) {
				my($title, $name, $args, $dot, $i, %args)=($2, $3, $4, qr(^\Q$5\E\s*$)o, 0);
				$name||=sprintf $vp, $vi;
				while($args=~/\s*(["']?)([^\1,\:]+)\1\s*(?:\:\s*(["']?)([^,]+)\3)?\s*,?/goc) {
					my$key=&RXParser::Filter::trim($2);
					$args{$key}={
						order=>++$i,
						type=>$4
					} unless $key eq undef
				}
				$self->{result}{$name}={
					order=>$vi++,
					title=>$title,
					args=>{%args}
				};
				while(<$fh>) {
					last if /$dot/o;
					s/^\s*(.*?)\s*$/\\s\*$1\\s\*/o;
					$self->{result}{$name}{rx}.=$_;
				}
				$self->{result}{$name}{rx}=~s/(?:\\s\*){2,}/\\s*/go;
				$self->{result}{$name}{rx}=~s/^(?:\\s\*)(.*?)(?:\\s\*)$/$1/go;
			}
		}
	&fileops::osclose($fh);
	+$self
}

+1;

package RXParser::Parser;
use base 'RXParser';

sub prepare {
	my$self=shift;
	my$mapping=shift  || $self->{mapping} || return undef;
	my$file=&fileops::osreadfile(shift || $self->{file} || return undef);
	my($q, $mr, $i, $key, $res, @keys)=(0, $mapping->{result});
	$self->{result}={};

	foreach$key(sort{$mr->{$a}{order}<=>$mr->{$b}{order}}keys%$mr) {
		$self->{result}{$key}={
			found=>[],
			info=>$mr->{$key}
		};
		$q=0;
		pos($file)=0;
		@keys=sort{$mr->{$key}{args}{$a}{order}<=>$mr->{$key}{args}{$b}{order}} keys%{$mr->{$key}{args}};
		while($file=~m/$mr->{$key}{rx}/giosc) {
			$res={
				'#'=>+$q++,
				'!'=>+pos $file
			};
			for($i=0; $i<@keys; $i++) {
				$res->{$keys[$i]}=RXParser::Filter->prepare(${$i+1}, $mr->{$key}{args}{$keys[$i]}{type})
			}
			push @{$self->{result}{$key}{found}}, $res
		}
	}
	+$self
}

+1;

package RXParser::RXML;
use base 'RXParser';

BEGIN {
	$PREFIX='rxml';
	%PREFIXES=(
		'iter'=>1
	);
}

sub explode {
	my$self=shift;
	+[$self->buildtree(&fileops::osreadfile(shift))]
}

sub buildnode {
	my$self=shift;
	my($str, $pos, @result)=shift;
	if($str=~s/<$PREFIX\:(["']?)([^>]+?)\1\s+([^>]+?)\s*(?:\/>|>\s*(.*?)\s*<\/$PREFIX\:(["']?)\2\5>)/
		push @result, substr($`, $pos, pos($str)-$pos), {
			content=>$4,
			node=>$2,
			params=>$self->getparams($3)
		};
		$pos=pos($str)+length($&);
		+undef
	/geios) {
		+ @result, $'
	} else {
		+ $str
	}
}

sub buildtree {
	my$self=shift;
	my($str, $pos, @result, $prefix, $content, $node, $params)=shift;
	my$len=length $str;
	if($str=~s/<$PREFIX(\-\w+)\:(["']?)([^>]+?)\2(?:([^>]+?))?>\s*(.*?)\s*<\/$PREFIX\1\:(["']?)\3\6>/
		if(exists $PREFIXES{+$prefix=lc substr $1, 1}) {
			($content, $node, $params)=($5, $3, $4);
			push @result, $self->buildnode(substr $`, $pos, pos($str)-$pos),
			{
				prefix=>$prefix,
				content=>$content,
				node=>$node,
				params=>$self->getparams($params)
			};
		}
		$pos=pos($str)+length($&);
		+undef
	/geios) {
		push @result, $';
		foreach(@result) {
			if(ref eq 'HASH') {
				if(defined $_->{content}) {
					$_->{content}=[$self->buildtree($_->{content})]
				} else {
					delete $_->{content}
				}
			} else {
				$_=[$self->buildnode($_)]
			}
		}
		+ @result
	} else {
		+ $self->buildnode($str)
	}
}

sub buildout {
	my$self=shift;
	my$tree=shift;
	my$parent=shift;
	my$index=shift;

	if(ref $tree eq 'ARRAY') {
		$self->buildout($_, $parent, $index) foreach @$tree;
	} elsif(ref $tree eq 'HASH') {

		my$parser=$self->{parser}{result};
		if(exists $parser->{$tree->{node}}) {
			if($tree->{prefix} eq 'iter') {

				my$cur=$parser->{$tree->{node}}{found};
				if(ref $parent eq 'HASH' && $parent->{prefix} eq 'iter') {
					my$par=$parser->{$parent->{node}}{found};

					if($index==$#$par) {
						for(my$i=0; $i<@$cur; $i++) {
							$self->buildout($tree->{content}, $tree, $i)
									if $par->[$index]{'!'} < $cur->[$i]{'!'}
						}
					} else {
						for(my$i=0; $i<@$cur; $i++) {
							$self->buildout($tree->{content}, $tree, $i)
									if $par->[$index]{'!'} < $cur->[$i]{'!'} &&
										$par->[$index+1]{'!'} > $cur->[$i]{'!'}
						}
					}
				} else {
					for(my$i=0; $i<@$cur; $i++) {
						$self->buildout($tree->{content}, $tree, $i)
					}
				}
			} else {
				$index=$tree->{params}{index} if length $tree->{params}{index};
				print RXParser::Filter->prepare(
					$parser->{$tree->{node}}{found}[$index]{$tree->{params}{value}},
					$tree->{params}{type}
				);
			}
		} else {
			print "\n", '<!--', $tree->{node}, '-->', "\n"
		}
	} else {
		print $tree
	}
}

sub prepare {
	my$self=shift;
	$self->{tree}=$self->explode($self->{file} || return undef);
	+$self
}

sub saveAs {
	my$self=shift;
	return undef unless $self->{tree};
	my$parser=$self->{parser} || return undef;
	my$fh=&fileops::osopen('>'.(shift || $self->{output} || return undef), 2);
	my$old=select $fh;
	$self->buildout($self->{tree});
	select $old;
	+&fileops::osclose($fh)
}

sub getparam {
	my($self, $str, $param)=(shift, shift, shift);
	return $2 if $str=~/\b\Q$_[0]\E\s*=\s*(["']?)(.*?)\1/ios;
	+undef
}

sub getparams {
	my($self, $str, %result)=(shift, shift);
	$str=~s/(\w+)\s*=\s*(["']?)(.*?)\2/$result{+lc $1}=$3; +undef/geos;
	+\%result
}

+1;

__END__
=head1 NAME
 RXParser - Universal document translator
=head1 AUTHOR
 Shatrow Alexey <cdch@yandex.ru>,
 Krasnodar, March 13, 2005 (c)
=cut