package RXParser::Template;
use base 'RXParser';

BEGIN {
	$PREFIX='rxml';
	%PREFIXES=(
		'iter'=>sub{
				my($self, $tree, $parent, $index, $parser, $i)=(
						shift, shift, shift, shift, shift, 0);
				my$cur=$parser->{$tree->{node}}{found};
				my$len=scalar @$cur;

				if(ref $parent eq 'HASH' && $parent->{prefix} eq 'iter') {
					my$par=$parser->{$parent->{node}}{found};

					if($index==$#$par) {
						for(; $i<$len; $i++) {
							$self->buildout($tree->{content}, $tree, $i)
									if $par->[$index]{'!'} < $cur->[$i]{'!'}
						}
					} else {
						for(; $i<$len; $i++) {
							$self->buildout($tree->{content}, $tree, $i)
									if $par->[$index]{'!'} < $cur->[$i]{'!'} &&
										$par->[$index+1]{'!'} > $cur->[$i]{'!'}
						}
					}
				} else {
					for(; $i<$len; $i++) {
						$self->buildout($tree->{content}, $tree, $i)
					}
				}

				+1
		}
	);
}

sub explode {
	my$self=shift;
	use fileops;
	+[$self->buildtree(&fileops::osreadfile(shift))]
}

sub buildnode {
	my$self=shift;
	my($str, $pos, @result)=shift;
	if($str=~s/<$PREFIX\:(["']?)([^>]+?)\1\s+([^>]+?)\s*(?:\/>|>\s*(.*?)\s*<\/$PREFIX\:(["']?)\2\5>)/
		push @result, substr($`, $pos, pos($str)-$pos), {
			content=>$4,
			node=>$2,
			params=>$self->getparams($3)
		};
		$pos=pos($str)+length($&);
		+undef
	/geios) {
		+ @result, $'
	} else {
		+ $str
	}
}

sub buildtree {
	my$self=shift;
	my($str, $pos, @result, $prefix, $content, $node, $params)=shift;
	my$len=length $str;
	if($str=~s/<$PREFIX(\-\w+)\:(["']?)([^>]+?)\2(?:([^>]+?))?>\s*(.*?)\s*<\/$PREFIX\1\:(["']?)\3\6>/
		if(exists $PREFIXES{+$prefix=lc substr $1, 1}) {
			($content, $node, $params)=($5, $3, $4);
			push @result, $self->buildnode(substr $`, $pos, pos($str)-$pos),
			{
				prefix=>$prefix,
				content=>$content,
				node=>$node,
				params=>$self->getparams($params)
			};
		}
		$pos=pos($str)+length($&);
		+undef
	/geios) {
		push @result, $';
		foreach(@result) {
			if(ref eq 'HASH') {
				if(defined $_->{content}) {
					$_->{content}=[$self->buildtree($_->{content})]
				} else {
					delete $_->{content}
				}
			} else {
				$_=[$self->buildnode($_)]
			}
		}
		+ @result
	} else {
		+ $self->buildnode($str)
	}
}

sub buildout {
	my($self, $tree, $parent, $index)=(shift, shift, shift, shift);

	if(ref $tree eq 'ARRAY') {
		$self->buildout($_, $parent, $index) foreach @$tree;
	} elsif(ref $tree eq 'HASH') {

		my$parser=$self->{parser}{result};
		if(exists $parser->{$tree->{node}}) {
			if(ref $PREFIXES{$tree->{prefix}} eq 'CODE') {
				$PREFIXES{$tree->{prefix}}($self, $tree, $parent, $index, $parser);
			} else {
				$index=$tree->{params}{index} if length $tree->{params}{index};
				print $self->filter->prepare(
					$parser->{$tree->{node}}{found}[$index]{$tree->{params}{value}},
					$tree->{params}{type}
				);
			}
		} else {
			print "\n", '<!--', $tree->{node}, '-->', "\n"
		}

	} else {
		print $tree
	}
}

sub prepare {
	my$self=shift;
	my$file=$self->{file} or $self->error('Input rules filename not found');
	$self->{tree}=$self->explode($file);
	+$self
}

sub saveAs {

	#     

	my($self, $args)=(shift, {@_});
	$self->{$_}||=delete $args->{$_} foreach qw'tree parser output';
	$self->{output}||=delete $args->{file};

	$self->error('Tree object is not defined') unless $self->{tree};
	$self->error('RXParser::Parser object is not defined') unless $self->{parser};
	$self->error('Output filename not found') unless $self->{output};

	use fileops;
	my$fh=&fileops::osopen('>'.$self->{output}, 2);
	my$old=select $fh;$self->buildout($self->{tree});select $old;
	+&fileops::osclose($fh)
}

sub getparams {
	#     :
	# name1="value1" name2="value2" name3="value3"...

	my($self, $str, %result)=(shift, shift);
	$str=~s/(\w+)\s*=\s*(["']?)(.*?)\2/$result{+lc $1}=$3; +undef/geos;
	+\%result
}

+1;